.dropdown-item-checked::before {
  position: absolute;
  float: right;
  right: 0.6rem;
  content: "✓";
  font-weight: 600;
}

#invalid-char {
  color: red;
}

#invalid-name {
  color: red;
}

.custom-select {
  width: -webkit-fill-available !important;
  font-size: 16px !important;
}

.pre-modal {
  background: #e9d2b4 !important;
}
